from __future__ import annotations

import os

from ews_core_config.config import EWSSettings


def test_config() -> None:
    old_prefix = EWSSettings.model_config["env_prefix"]
    try:
        EWSSettings.model_config.update(env_prefix="XYZ_")
        settings = EWSSettings()
        settings.export_env_vars()
        envs = {k: v for k, v in os.environ.items() if k.startswith("XYZ_")}
        assert envs

    finally:
        EWSSettings.model_config.update(env_prefix=old_prefix)


def test_user_file_read():
    settings = EWSSettings()
    assert settings.sharepoint.username != ""



""" """

from __future__ import annotations

import logging
import os
from pathlib import Path

import tomlkit
from pydantic import Field
from pydantic_settings import (
    BaseSettings,
    PydanticBaseSettingsSource,
    SettingsConfigDict,
    TomlConfigSettingsSource,
)

import ews_core_config.schema as schema

log = logging.getLogger(__name__)

envs_prefix = os.getenv("EWS_ENVS_PREFIX", "EWS_")
config_path = os.getenv("EWS_CONFIG_PATH", Path.home() / ".config" / "ews")
config_file_name = os.getenv("EWS_CONFIG_FILENAME", "config")
if config_file_name.endswith(".toml"):
    config_file_name = config_file_name[:-4]

toml_file = Path(config_path) / f"{config_file_name}.toml"


class EWSSettings(BaseSettings, case_sensitive=False):
    paths: schema.Paths = Field(default_factory=schema.Paths)
    sharepoint: schema.SharePoint = Field(default_factory=schema.SharePoint)
    github: schema.GitHub = Field(default_factory=schema.GitHub)

    # Online documentation
    ewstools: schema.EwsTools = Field(default_factory=schema.EwsTools)

    # WEB Apis
    ammonitor: schema.WindCubeInsights = Field(default_factory=schema.WindCubeInsights)
    windcubeinsights: schema.WindCubeInsights = Field(default_factory=schema.WindCubeInsights)

    # Wheelhouse
    pyserver: dict[str, schema.PyServer] = Field(default_factory=lambda: schema.PyServers().model_dump())
    devpi: dict[str, schema.DevPi] = Field(default_factory=lambda: schema.DevPis().model_dump())

    model_config = SettingsConfigDict(
        toml_file=toml_file,
        extra="allow",
        env_prefix=envs_prefix,
        env_nested_delimiter="_",
        case_sensitive=False,
        frozen=True,
    )

    @classmethod
    def settings_customise_sources(
        cls,
        settings_cls: type[BaseSettings],
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> tuple[PydanticBaseSettingsSource, ...]:
        return (
            env_settings,
            dotenv_settings,
            TomlConfigSettingsSource(settings_cls),
            init_settings,
        )

    def create_env_vars(self) -> dict[str, str]:
        d = self.model_dump(mode="python")
        res = {}

        def make_env_var(value, parent_key=None) -> None:
            if isinstance(value, list):
                for i, val in enumerate(value):
                    k = (*parent_key, str(i)) if parent_key else (str(i),)
                    make_env_var(val, parent_key=k)
            elif isinstance(value, dict):
                for key, val in value.items():
                    k = (*parent_key, str(key)) if parent_key else (str(key),)
                    make_env_var(val, parent_key=k)
            else:
                res[parent_key] = str(value)

        make_env_var(d)
        r = {}
        env_prefix = self.model_config["env_prefix"]
        env_nested_delimiter = self.model_config["env_nested_delimiter"]
        for k, v in res.items():
            key = f"{env_prefix}{env_nested_delimiter.join(k)}"
            r[key.upper()] = v
        return r

    def export_env_vars(self) -> None:
        envs = self.create_env_vars()
        os.environ.update(envs)

    @property
    def toml_file(self) -> str:
        return str(self.model_config.get("toml_file", ""))

    def write_to_file(self, force: bool = False, **dumps_kws) -> None:
        out_file = self.model_config.get("toml_file", None)
        if not out_file:
            return
        out_file = Path(out_file)
        if out_file.is_file():
            if not force:
                raise RuntimeError(f"File {out_file!s} is not writable. Use force=True to allow")
        else:
            log.info(f"Creating {out_file!s}")
        data = tomlkit.dumps(self.model_dump(**dumps_kws))
        out_file.write_text(data)


def set_env_vars() -> None:
    EWSSettings().export_env_vars()


__all__ = ("set_env_vars", "EWSSettings")



from __future__ import annotations

import platform
import sys
from importlib import metadata
from pathlib import Path

# Do not use the namespace package!!!
VERSION = metadata.version("ews-core-config")


def _get_sys_info() -> dict[str, str]:
    """System information

    Returns:
        system and Python version information
    """

    blob = [
        ("python", sys.version.replace("\n", " ")),
        ("executable", sys.executable),
        ("platform", platform.platform()),
    ]

    return dict(blob)


def _get_deps_info() -> dict[str, str]:
    """Overview of the installed version of main dependencies

    Returns:
        version information on relevant Python libraries
    """
    deps = [
        "ews-core-config",
        "pydantic",
        "tomlkit",
        "click",
    ]
    deps_info = {}

    for modname in deps:
        try:
            deps_info[modname] = metadata.version(modname)
        except ModuleNotFoundError:  # pragma: no cover
            deps_info[modname] = ""

    return deps_info


def show_versions() -> None:
    """
    Show the versions of the package with its optional dependencies

    Example:
        ```python
        import ews_core_config

        ews_core_config.show_versions()
        ```

    """
    sys_info: dict[str, str] = _get_sys_info()
    deps_info: dict[str, str] = _get_deps_info()

    maxlen = max(len(x) for x in sys_info)
    if deps_info:
        maxlen = max(maxlen, *(len(x) for x in deps_info))
    else:  # pragma: no cover
        pass
    maxlen += 2

    tpl: str = f"{{k:<{maxlen}}}: {{stat}}"
    print("\nSYSTEM INFO")
    print("-----------")
    for k, stat in sys_info.items():
        print(tpl.format(k=k, stat=stat))

    if deps_info:
        print("\nPYTHON DEPENDENCIES")
        print("-------------------")
        for k, stat in deps_info.items():
            print(tpl.format(k=k, stat=stat))


def version_info() -> str:
    """
    Show the version info

    Example:
        ```python
        import ews_core_config as ews_cfg

        print(ews_cfg.version_info())
        ```

    """
    sys_info: dict[str, str] = _get_sys_info()
    deps_info: dict[str, str] = _get_deps_info()
    optional_deps: list[str] = [f"{k}=={v}" for k, v in deps_info.items() if k]

    info = {
        "ews_core_config version": VERSION,
        "install path": Path(__file__).resolve().parent,
        "python": sys_info["python"],
        "platform": sys_info["platform"],
        "optional deps. installed": optional_deps,
    }
    return "\n".join("{:>30} {}".format(k + ":", str(v).replace("\n", " ")) for k, v in info.items())


__all__ = ("VERSION", "show_versions", "version_info")



""" """

from __future__ import annotations

import socket
import subprocess
import textwrap

from pydantic import BaseModel, ConfigDict, Field

default_mappings = "".join(
    map(
        textwrap.dedent,
        r"""
    /ews.local/GLOGBAL/Daten:f;
    /ews.local/GLOBAL/QM:p;
    /SRFILEWS001.ews.local/daten$:f;
    /SRFILEWS001.ews.local/qm$:p;
    /SXFILEWS001.ews.local/roh_dat$:r;
    /smuffile001/daten$:f;
    /smuffile001/qm$:p;
    /smuffile001/ROHDATEN$:r
    /spanfile001/daten$:f;
    /spanfile001/qm$:p;
    /smuffile001/ROHDATEN$:r
    """.strip().split("\n"),
    )
)


hostname = socket.gethostname().upper()

name = email = gh_username = ""
try:
    p = subprocess.run(
        "git config --get user.name",  # noqa: S607
        shell=True,  # noqa: S602
        capture_output=True,
        text=True,
        check=False,
    )
    if not p.returncode:
        name = p.stdout.strip()
    p = subprocess.run(
        "git config --get user.email",  # noqa: S607
        shell=True,  # noqa: S602
        capture_output=True,
        text=True,
        check=False,
    )
    if not p.returncode:
        email = p.stdout.strip()
        gh_username = "ews-" + email.split("@")[0].replace(".", "")
except Exception as e:
    raise e
finally:
    pass


class FrozenModel(BaseModel):
    model_config = ConfigDict(frozen=True, strict=False, validate_default=True, defer_build=True)


class Paths(FrozenModel):
    mount_point: str = "/mnt"
    drives: str = "c;d;f;p;r"
    mappings: str = default_mappings


class SharePoint(FrozenModel):
    username: str = email
    password: str = ""
    site: str = "https://ewsconsulting.sharepoint.com/sites/teams_mb/"
    server: str = "ewsconsulting.sharepoint.com"


class GitHub(FrozenModel):
    username: str = gh_username
    token: str = ""


class EwsTools(FrozenModel):
    url: str = "https://mb.ews.tools"
    host: str = "smufcm.ews.local"
    port: int = 52812
    username: str = "mb_upload"
    password: str = ""


class AmmonitOR(FrozenModel):
    username: str = "wm@ews-consulting.com"
    password: str = ""


class WindCubeInsights(FrozenModel):
    username: str = "wm@ews-consulting.at"
    password: str = ""


class DevPi(FrozenModel):
    hostname: str = hostname
    port: int = 8051
    index: str = "root/private"
    url: str = r"root/all/+simple"


class PyServer(FrozenModel):
    hostname: str = ""
    port: int = 8050
    index: str = ""
    url: str = "simple"


class PyServers(FrozenModel):
    local: PyServer = Field(default_factory=lambda: PyServer(hostname=hostname))
    remote: PyServer = Field(default_factory=PyServer)


class DevPis(FrozenModel):
    local: DevPi = Field(default_factory=lambda: DevPi(hostname=hostname))
    remote: DevPi = Field(default_factory=DevPi)



from __future__ import annotations

import io
from pathlib import Path
from pprint import pprint

import click

from ews_core_config.config import EWSSettings
from ews_core_config.version import VERSION


@click.group()
@click.version_option(VERSION, "-v", "--version", is_flag=True, help="Show the version")
def cli():
    pass


def show_dict_as_env(d: dict, export: bool = False, **kwargs):
    if not d:
        return
    p = "export " if export else ""
    for k, v in d.items():
        txt = f'{p}{k.upper()}="{v}"'
        click.secho(txt, **kwargs)


def show_dict_as_text(d: dict, min_pad: int = 20, **kwargs):
    if not d:
        return
    max_pad = max(map(len, d.keys())) + 3
    max_pad = max(min_pad, max_pad)
    for k, v in d.items():
        txt = f"{k.ljust(max_pad)}: {v}"
        click.secho(txt, **kwargs)


def pprint_dict(d: dict, **kwargs):
    if not d:
        return
    stream = io.StringIO()
    pprint(d, stream=stream)
    stream = stream.getvalue()
    click.secho(stream, **kwargs)


@cli.command(name="show", help="Show the config attributes and values")
@click.option("-t/-nt", "--text/--no-text", is_flag=True, default=True, show_default=True)
@click.option("-l/-nl", "--location/--no-location", is_flag=True, default=True, show_default=True)
def show(text: bool = True, location: bool = True):
    settings = EWSSettings()
    cfg_filename = Path(settings.toml_file)
    if location:
        click.secho(f"Location: {cfg_filename!s}\n", fg="yellow", bold=True)

    nice = settings.model_dump(mode="python")
    if text:
        show_dict_as_text(nice, fg="blue")
    else:
        pprint_dict(nice, fg="blue")


@cli.command(name="env", help="Show the set environment variables")
@click.option("-t/-nt", "--text/--no-text", is_flag=True, default=True, show_default=True)
@click.option("-e/-ne", "--env/--no-env", is_flag=True, default=False, show_default=True)
@click.option("--export", is_flag=True, default=False, show_default=True)
@click.option("-l/-nl", "--location/--no-location", is_flag=True, default=True, show_default=True)
def env(text: bool = True, env: bool = False, export: bool = False, location: bool = True):
    settings = EWSSettings()

    if env or export:
        text = location = False
        env = True

    cfg_filename = Path(settings.toml_file)
    if location:
        click.secho(f"Location: {cfg_filename!s}\n", fg="yellow", bold=True)
    env_vars = settings.create_env_vars()
    if text:
        show_dict_as_text(env_vars, fg="blue")
    elif env:
        show_dict_as_env(env_vars, export=export, fg="blue")
    else:
        pprint_dict(env_vars, fg="blue")


@cli.command(name="path", help="Show the path of the config file")
def _path() -> None:
    settings = EWSSettings()
    cfg_filename = Path(settings.toml_file)
    click.echo(f"{cfg_filename!s}")


@cli.command(name="init", help="Create a new config file if it does not exist")
@click.option("--verbose/--quiet", is_flag=True, default=True, show_default=True)
def _init(verbose: bool) -> None:
    settings = EWSSettings()
    settings.write_to_file(force=False, mode="python")
    cfg_filename = Path(settings.toml_file)
    if verbose:
        click.echo(f"Created {cfg_filename!s}")


@cli.command(name="update", help="Update a config file if it does not exist")
@click.option("--verbose/--quiet", is_flag=True, default=False, show_default=True)
def _update(verbose: bool) -> None:
    settings = EWSSettings()
    cfg_filename = Path(settings.toml_file)
    settings.write_to_file(force=True)
    if verbose:
        click.echo(f"{cfg_filename!s}")
